from twisted.trial import unittest

from twisted.internet import defer, reactor, error
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol

from tdjango import DjangoORM

class Test(unittest.TestCase):

    @defer.inlineCallbacks
    def setUp(self):
        self.db = DjangoORM('tdjango.tests.testapp')

        yield self.db.delete('auth_user')

        yield self.db.Color.objects.create(
            color='red', 
            r=255
        ).save()

        yield self.db.Color.objects.create(
            color='blue', 
            b=255
        ).save()

        yield self.db.User.objects.create(
            username='john'
        ).save()


    @defer.inlineCallbacks
    def tearDown(self):
        # Truncate all the tables
        yield self.db.delete('testapp_rainbows_colors')
        yield self.db.delete('testapp_rainbows')
        yield self.db.delete('testapp_animals')
        yield self.db.delete('testapp_color')
        yield self.db.delete('auth_user')

    def test_adapter(self):
        self.assertIn('Color', self.db.models)
        self.assertIn('Animals', self.db.models)
        self.assertIn('User', self.db.models)
        
    @defer.inlineCallbacks
    def test_create(self):
        model = self.db.Color.objects.create(
            color='green', 
            g=255
        )

        self.assertEquals(model.g, 255)
        self.assertEquals(model.color, 'green')

        yield model.save()

        m2 = yield self.db.Color.objects.get(color='green')
        self.assertEquals(m2.g, 255)
        self.assertEquals(m2.r, 0)
    
    @defer.inlineCallbacks
    def test_update(self):
        blue = yield self.db.Color.objects.get(color='blue')

        blue.g = 30

        yield blue.save()

        blue = yield self.db.Color.objects.get(color='blue')

        self.assertEquals(blue.g, 30)
        self.assertEquals(blue.b, 255)
        self.assertEquals(blue.r, 0)

    @defer.inlineCallbacks
    def test_relations(self):
        red = yield self.db.Color.objects.get(color='red')
        john = yield self.db.User.objects.get(username='john')
        
        snake = self.db.Animals.objects.create(
            name='Red snake',
            weight=1,
            owner=john,
            color=red
        )

        yield snake.save()

        id = snake.id

        mysnake = yield self.db.Animals.objects.get(name='Red snake')
        
        self.assertEquals(mysnake.id, id)
        self.assertEquals(mysnake.owner, john)
        self.assertEquals(mysnake.color, red)
        self.assertEquals(mysnake.color.id, red.id)
        self.assertEquals(mysnake.color.r, 255)

    @defer.inlineCallbacks
    def test_manymany(self):
        red = yield self.db.Color.objects.get(color='red')
        blue = yield self.db.Color.objects.get(color='blue')
        green = self.db.Color.objects.create(
            color='green', 
            g=255
        )
        green.save()

        rainbow = self.db.Rainbows.objects.create(
            name='rgb'
        )

        yield rainbow.save()

        yield rainbow.colors.add(red)

        self.assertIn(red, rainbow.colors)

        yield rainbow.colors.set([green, blue])

        yield rainbow.save()

        rain2 = yield self.db.Rainbows.objects.get(name='rgb')

        self.assertIn(green, rain2.colors)
        self.assertIn(blue, rain2.colors)

        yield rain2.delete()
        
    @defer.inlineCallbacks
    def test_objects_filter(self):
        colors = yield self.db.Color.objects.filter(color='red')

        red = yield self.db.Color.objects.get(color='red')
        self.assertIn(red, colors)

    @defer.inlineCallbacks
    def test_objects_all(self):
        colors = yield self.db.Color.objects.all()

        red = yield self.db.Color.objects.get(color='red')
        self.assertIn(red, colors)





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('testapp', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Rainbows',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('colors', models.ManyToManyField(to='testapp.Color')),
            ],
        ),
        migrations.RemoveField(
            model_name='animals',
            name='owners',
        ),
        migrations.AddField(
            model_name='animals',
            name='owner',
            field=models.ForeignKey(default=1, blank=True, to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
    ]



from twisted.internet import defer


class DBMixin(object):
    def _fetchOneTxn(self, txn, *a, **kw):
        " Transaction callback for self.fetchOne "
        txn.execute(*a)

        r = txn.fetchall()

        if r:
            return r[0]
        else:
            return None

    def fetchOne(self, *a, **kw):
        " Fetch one row only with this query "
        return self.p.runInteraction(self._fetchOneTxn, *a, **kw)

    
    def runInsert(self, table, keys):
        " Builds a simple INSERT statement"
        # Unzip the items tupple set into matched order k/v's
        keys, values = zip(*keys.items())

        st = "INSERT INTO %s (%s) VALUES (%s) RETURNING id" % (
            table,
            ','.join(keys),
            ','.join(['%s']*len(keys)) # Witchcraft
        )

        return self.fetchOne(st, values)

    def runUpdate(self, table, keys, **constraints):
        " Builds a simple UPDATE statement"
        setkeys, setvalues = zip(*keys.items())
        ckeys, cvalues = zip(*constraints.items())

        st = "UPDATE %s SET %s" % (
            table,
            ','.join(['%s=%%s' % (k) for k in setkeys])
        )

        if ckeys:
            st += " WHERE %s" % (' AND '.join(['%s=%%s' % (k) for k in ckeys]))

        return self.p.runOperation(st, setvalues + cvalues)

    def delete(self, table, **constraints):
        st = 'DELETE FROM %s ' % table

        if constraints:
            ckeys, cvalues = zip(*constraints.items())
            if ckeys:
                st += " WHERE %s" % (' AND '.join(['%s=%%s' % (k) for k in ckeys]))
        else:
            cvalues = []
        
        return self.p.runOperation(st, cvalues)

    @defer.inlineCallbacks
    def select(self, table, fields, **kw):
        q = []
        args = []
        for k, v in kw.items():
            q.append('%s=%%%%s' % k)
            args.append(v)

        query = "SELECT %s FROM %s" 

        if q:
            query += " WHERE " + ' and '.join(q)

        results = yield self.p.runQuery(query % (
                ','.join(fields),
                table,
            ), tuple(args))

        res = []

        for r in results:
            obj = {}
            for i, col in enumerate(r):
                obj[fields[i]] = col

            res.append(obj)

        defer.returnValue(res)

    @defer.inlineCallbacks
    def selectOne(self, *a, **kw):
        r = yield self.select(*a, **kw)
        if r:
            defer.returnValue(r[0])
        else:
            defer.returnValue(None)




from django.db import models

from django.contrib.auth.models import User

class Color(models.Model):
    color = models.CharField(max_length=255)
    r, g, b = (models.IntegerField(default=0), models.IntegerField(default=0),
        models.IntegerField(default=0))

class Animals(models.Model):
    name = models.CharField(max_length=255, unique=True)
    weight = models.IntegerField()
    owner = models.ForeignKey(User, blank=True)
    
    color = models.ForeignKey(Color)

class Rainbows(models.Model):
    name = models.CharField(max_length=255, unique=True)
    colors = models.ManyToManyField(Color)

